@@ -1,9 +1,9 @@
 #!/bin/bash
 #		GMT EXAMPLE 29
-#		$Id: example_29.sh 16652 2016-06-28 21:02:26Z pwessel $
+#		$Id: example_29.sh 17806 2017-03-27 22:18:01Z pwessel $
 #
 # Purpose:	Illustrates spherical surface gridding with Green's function of splines
-# GMT modules:	makecpt, grdcontour, grdgradient, grdimage, grdmath greenspline, psscale, pstext
+# GMT modules:	makecpt, grdcontour, grdimage, grdmath greenspline, psscale, pstext
 # Unix progs:	rm, echo
 #
 ps=example_29.ps
@@ -26,18 +26,16 @@ gmt greenspline -RPROJ_ELLIPSOID.nc mars370.txt -D4 -Sq0.9975 -Gmars2.nc
 gmt grdmath mars.nc 1000 DIV PROJ_ELLIPSOID.nc SUB = mars.nc
 gmt grdmath mars2.nc 1000 DIV PROJ_ELLIPSOID.nc SUB = mars2.nc
 gmt makecpt -Crainbow -T-7/15 > mars.cpt
-gmt grdgradient mars2.nc -fg -Ne0.75 -A45 -Gmars2_i.nc
-gmt grdimage mars2.nc -Imars2_i.nc -Cmars.cpt -B30g30 -BWsne -JH0/7i -P -K -E200 \
+gmt grdimage mars2.nc -I+ne0.75+a45 -Cmars.cpt -B30g30 -BWsne -JH0/7i -P -K -E200 \
 	--FONT_ANNOT_PRIMARY=12p -X0.75i > $ps
 gmt grdcontour mars2.nc -J -O -K -C1 -A5 -Glz+/z- >> $ps
 gmt psxy -Rg -J -O -K -Sc0.045i -Gblack mars370.txt  >> $ps
 echo "0 90 b)" | gmt pstext -R -J -O -K -N -D-3.5i/-0.2i -F+f14p,Helvetica-Bold+jLB >> $ps
-gmt grdgradient mars.nc -fg -Ne0.75 -A45 -Gmars_i.nc
-gmt grdimage mars.nc -Imars_i.nc -Cmars.cpt -B30g30 -BWsne -J -O -K -Y4.2i -E200 \
+gmt grdimage mars.nc -I+ne0.75+a45 -Cmars.cpt -B30g30 -BWsne -J -O -K -Y4.2i -E200 \
 	--FONT_ANNOT_PRIMARY=12p >> $ps
 gmt grdcontour mars.nc -J -O -K -C1 -A5 -Glz+/z- >> $ps
 gmt psxy -Rg -J -O -K -Sc0.045i -Gblack mars370.txt  >> $ps
-gmt psscale -Cmars.cpt -O -K -R -J -DJBC+o0/0.15i+w6i/0.1i+h -I --FONT_ANNOT_PRIMARY=12p -Bx2f1 -By+lkm >> $ps
+gmt psscale -Cmars.cpt -O -K -R -J -DJBC+o0/0.15i+w6i/0.1i -I --FONT_ANNOT_PRIMARY=12p -Bx2f1 -By+lkm >> $ps
 echo "0 90 a)" | gmt pstext -R -J -O -N -D-3.5i/-0.2i -F+f14p,Helvetica-Bold+jLB >> $ps
 # Clean up
 rm -f *.nc mars.cpt
