@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_macros.h 16722 2016-07-06 13:46:09Z remko $
+ *	$Id: gmt_macros.h 18041 2017-04-26 23:09:48Z pwessel $
  *
- *	Copyright (c) 1991-2016 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * gmt_macros.h contains definitions of macros used throught GMT.
+ * gmt_macros.h contains definitions of macros used through GMT.
  *
  * Author:	Paul Wessel
  * Date:	01-OCT-2009
@@ -25,7 +25,7 @@
 
 /*!
  * \file gmt_macros.h
- * \brief Definitions of macros used throught GMT.
+ * \brief Definitions of macros used through GMT.
  */
 
 #ifndef _GMT_MACROS_H
@@ -89,13 +89,13 @@
 
 /*! Macros for swapping misc data types */
 
-#define bool_swap(x, y) {bool bool_tmp; bool_tmp = x, x = y, y = bool_tmp;}
-#define char_swap(x, y) {char char_tmp; char_tmp = x, x = y, y = char_tmp;}
-#define uint64_swap(x, y) {uint64_t uint64_t_tmp; uint64_t_tmp = x, x = y, y = uint64_t_tmp;}
-#define int_swap(x, y) {int int_tmp; int_tmp = x, x = y, y = int_tmp;}
-#define uint_swap(x, y) {unsigned int uint_tmp; uint_tmp = x, x = y, y = uint_tmp;}
-#define double_swap(x, y) {double double_tmp; double_tmp = x, x = y, y = double_tmp;}
-#define float_swap(x, y) {float float_tmp; float_tmp = x, x = y, y = float_tmp;}
+#define gmt_M_bool_swap(x, y) {bool bool_tmp; bool_tmp = x, x = y, y = bool_tmp;}
+#define gmt_M_char_swap(x, y) {char char_tmp; char_tmp = x, x = y, y = char_tmp;}
+#define gmt_M_uint64_swap(x, y) {uint64_t uint64_t_tmp; uint64_t_tmp = x, x = y, y = uint64_t_tmp;}
+#define gmt_M_int_swap(x, y) {int int_tmp; int_tmp = x, x = y, y = int_tmp;}
+#define gmt_M_uint_swap(x, y) {unsigned int uint_tmp; uint_tmp = x, x = y, y = uint_tmp;}
+#define gmt_M_double_swap(x, y) {double double_tmp; double_tmp = x, x = y, y = double_tmp;}
+#define gmt_M_float_swap(x, y) {float float_tmp; float_tmp = x, x = y, y = float_tmp;}
 
 /*! Macro to ensure proper value and sign of a change in longitude from lon1 to lon2 */
 #define gmt_M_set_delta_lon(lon1,lon2,delta) {delta = lon2 - lon1; if (fabs (delta) > 180.0) delta = copysign (360.0 - fabs (delta), -delta);}
@@ -109,8 +109,8 @@
 #define gmt_M_cpy3v(to,from) memcpy(to, from, 3*sizeof(double))
 
 /*! Macros for printing a tic/toc elapsed time message*/
-#define gmt_M_tic(C) {if (C->current.setting.verbose >= GMT_MSG_TICTOC) GMT_Message(C->parent,GMT_TIME_RESET,"");}
-#define gmt_M_toc(C,...) {if (C->current.setting.verbose >= GMT_MSG_TICTOC) GMT_Message(C->parent,GMT_TIME_ELAPSED, \
+#define gmt_M_tic(C) {if (C->current.setting.verbose == GMT_MSG_TICTOC) GMT_Message(C->parent,GMT_TIME_RESET,"");}
+#define gmt_M_toc(C,...) {if (C->current.setting.verbose == GMT_MSG_TICTOC) GMT_Message(C->parent,GMT_TIME_ELAPSED, \
 		"(%s) | %s\n", C->init.module_name, __VA_ARGS__);}
 
 /*! Cleaner check to see if a line is associated with the extended syntax or not */
@@ -118,6 +118,11 @@
 
 /* COLOR MACROS */
 
+/* Determine if fill is in fact a pattern */
+/* Old: always starts with integer dpi.
+ * New: Either start with pattern 1-88 or a file which should have an extension */
+#define gmt_M_is_pattern(txt) ((txt[0] == 'p' || txt[0] == 'P') && (isdigit((int)txt[1]) || strchr(txt,'.')))
+
 /*! Copy two RGB[T] arrays (a = b) */
 #define gmt_M_rgb_copy(a,b) memcpy (a, b, 4 * sizeof(double))
 
@@ -153,4 +158,9 @@
 /*! Determine default justification for box item */
 #define gmt_M_just_default(GMT,refpoint,just) (refpoint->mode == GMT_REFPOINT_JUST_FLIP ? gmt_flip_justify(GMT,refpoint->justify) : refpoint->mode == GMT_REFPOINT_JUST ? refpoint->justify : just)
 
+/*! Determine if we have a special downloadable file */
+#define gmt_M_file_is_cache(file) (file && file[0] == '@' && strncmp (file, "@GMTAPI@-", 9U))
+#define gmt_M_file_is_url(file) (!strncmp (file, "http:", 5U) || !strncmp (file, "https:", 6U) || !strncmp (file, "ftp:", 4U))
+
+
 #endif  /* _GMT_MACROS_H */
