@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- * $Id: gmt_notposix.h 15757 2016-02-24 23:25:49Z pwessel $
+ * $Id: gmt_notposix.h 17937 2017-04-15 02:51:15Z pwessel $
  *
- * Copyright (c) 1991-2016 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis, and F. Wobbe
+ * Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis, and F. Wobbe
  * See LICENSE.TXT file for copying and redistribution conditions.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -199,6 +199,10 @@
 #	define PATH_SEPARATOR ':' /* Win uses ; while Unix uses : */
 #endif
 
+#ifndef S_ISDIR
+#	define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR) 	/* Copied from linux libc sys/stat.h */
+#endif
+
 /* Misc. ANSI-C math functions used by grdmath and gmtmath.
  * These functions are available on many platforms and we
  * seek to use them.  If not available then we compile in
@@ -485,6 +489,13 @@
 #	define fileno _fileno
 #endif
 
+/* rmdir is usually in unistd.h; we use a macro here
+ * since the same function under WIN32 is prefixed with _
+ * and defined in direct.h */
+#ifdef HAVE__GETCWD
+#	define rmdir _rmdir
+#endif
+
 /* getcwd is usually in unistd.h; we use a macro here
  * since the same function under WIN32 is prefixed with _
  * and defined in direct.h */
@@ -492,6 +503,13 @@
 #	define getcwd _getcwd
 #endif
 
+/* mktemp is usually in unistd.h; we use a macro here
+ * since the same function under WIN32 is prefixed with _
+ * and defined in io.h */
+#if defined HAVE__MKTEMP && !defined HAVE_MKTEMP
+#	define mktemp _mktemp
+#endif
+
 /* getpid is usually in unistd.h; we use a macro here
  * since the same function under WIN32 is prefixed with _
  * and defined in process.h */
