@@ -1,7 +1,7 @@
 /*
- *	$Id: gmtconnect.c 16996 2016-08-24 00:20:46Z jluis $
+ *	$Id: gmtconnect.c 18134 2017-05-05 08:34:43Z pwessel $
  *
- *	Copyright (c) 1991-2016 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,14 +27,14 @@
  * between segment endpoints exceeds a specified threshold.
  */
 
+#include "gmt_dev.h"
+
 #define THIS_MODULE_NAME	"gmtconnect"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Connect individual lines whose end points match within tolerance"
 #define THIS_MODULE_KEYS	"<D{,>D},CD),LT),QT)"
-
-#include "gmt_dev.h"
-
-#define GMT_PROG_OPTIONS "-:>Vabdfghios" GMT_OPT("HMm")
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "-:>Vabdefghios" GMT_OPT("HMm")
 
 /* Control structure for gmtconnect */
 
@@ -118,8 +118,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
 	GMT_Message (API, GMT_TIME_NONE, "usage: gmtconnect [<table>] [-C<closedfile>] [-D[<template>]] [-L[<linkfile>]] [-Q<listfile>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T[%s[/<nn_dist>]] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n",
-		GMT_DIST_OPT, GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t-T[%s[/<nn_dist>]] [%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n",
+		GMT_DIST_OPT, GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -142,7 +142,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   If no arguments are given the we close all polygons regardless of the gaps.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Used with -D to write names of files to a list.  Optionally give listfile name [gmtconnect_list.txt].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Embed %%c in the list name to write two separate lists: one for C(losed) and one for O(pen).\n");
-	GMT_Option (API, "a,bi2,bo,d,f,g,h,i,o,s,:,.");
+	GMT_Option (API, "a,bi2,bo,d,e,f,g,h,i,o,s,:,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -295,8 +295,8 @@ int GMT_gmtconnect (void *V_API, int mode, void *args) {
 
 	/* Parse the command-line arguments */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);		/* Allocate and initialize defaults in a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
@@ -443,7 +443,7 @@ int GMT_gmtconnect (void *V_API, int mode, void *args) {
 				segment[id].group = tbl;	/* Remember which input table this segment came from */
 				segment[id].pos = seg;		/* Remember which input segment in this table it came from */
 				segment[id].n = np;		/* Number of points in this segment */
-				/* Record start and end coordinates for this segment and initialze buddy structure to having no nearest neighbor segment yet */
+				/* Record start and end coordinates for this segment and initialize buddy structure to having no nearest neighbor segment yet */
 				segment[id].x_end[END_A] = S->data[GMT_X][0];
 				segment[id].y_end[END_A] = S->data[GMT_Y][0];
 				segment[id].x_end[END_B] = S->data[GMT_X][np-1];
@@ -687,7 +687,7 @@ int GMT_gmtconnect (void *V_API, int mode, void *args) {
 				b_alloc <<= 1;
 				buffer = gmt_M_memory (GMT, buffer, b_alloc, char);
 			}
-			strcat (buffer, text);	/* Append this connnection */
+			strcat (buffer, text);	/* Append this connection */
 			if (id2 == start_id)	/* Ended up at the starting polygon so it is now a closed polygon */
 				done = closed = true;
 			if (id2 == id || n_steps_pass_1 > ns) {	/* Not good... [NOT SURE WHAT THIS MEANS BUT SEEMS LIKE A CRAZY SAFETY VALVE AND SHOULD NEVER HAPPEN] */
@@ -769,7 +769,6 @@ int GMT_gmtconnect (void *V_API, int mode, void *args) {
 		p_first_x = p_last_x = DBL_MAX;
 		p_first_y = p_last_y = DBL_MAX;
 		n_steps_pass_2 = out_p = n_seg_length = 0;
-		done = false;
 		first = true;
 		do {
 			G = segment[id].group;	/* This is which table this line segment came from */
