@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------
- *	$Id: mgd77_init.h 15757 2016-02-24 23:25:49Z pwessel $
+ *	$Id: mgd77_init.h 17497 2017-01-28 23:57:39Z pwessel $
  *
- *    Copyright (c) 2005-2016 by P. Wessel
+ *    Copyright (c) 2005-2017 by P. Wessel
  *    See README file for copying and redistribution conditions.
  *
  *  File:	mgd77_init.h
@@ -12,7 +12,7 @@
  *		Michael Chandler, Affiliate Researcher, SOEST, U. of Hawaii
  *		
  *  This include file contains initializations for the MGD77 system.
- *  MUST BE INCUDED AFTER mgd77.h IN mgd77.c
+ *  MUST BE INCLUDED AFTER mgd77.h IN mgd77.c
  *
  *  Version:	1.1
  *  Revised:	1-JAN-2006
@@ -26,4 +26,4 @@
 
 bool MGD77_format_allowed[MGD77_N_FORMATS] = {true, true, true, true};	/* By default we allow opening of files in any format.  See MGD77_Ignore_Format() */
 
-char *MGD77_suffix[MGD77_N_FORMATS] = {"mgd77", "nc", "dat", "m77t"};
+char *MGD77_suffix[MGD77_N_FORMATS] = {"nc", "m77t", "mgd77", "dat"};
