@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: talwani2d.c 16555 2016-06-16 22:49:46Z pwessel $
+ *	$Id: talwani2d.c 18134 2017-05-05 08:34:43Z pwessel $
  *
- *	Copyright (c) 1991-2016 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -29,30 +29,27 @@
  *
  * Chapman, M. E. (1979), Techniques for interpretation of geoid
  *    anomalies, J. Geophys. Res., 84(B8), 3793–3801.
+ * Kim, S.-S., and P. Wessel, New analytic solutions for modeling
+ *    vertical gravity gradient anomalies, Geochem. Geophys. Geosyst.,
+ *    17, 2016, doi:10.1002/2016GC006263. [for VGG]
  * Rasmussen, R., and L. B. Pedersen (1979), End corrections in
  *   potential field modeling, Geophys. Prospect., 27, 749–760.
  * Talwani, M., J. L. Worzel, and M. Landisman (1959), Rapid gravity
  *    computations for two-dimensional bodies with application to
  *    the Mendocino submarine fracture zone, J. Geophys. Res., 64, 49–59.
- * Kim, S.-S., and P. Wessel, 2015, in prep.
  *
  * Accelerated with OpenMP; see -x.
  */
 
+#include "gmt_dev.h"
+#include "talwani.h"
+
 #define THIS_MODULE_NAME	"talwani2d"
 #define THIS_MODULE_LIB		"potential"
 #define THIS_MODULE_PURPOSE	"Compute geopotential anomalies over 2-D bodies by the method of Talwani"
 #define THIS_MODULE_KEYS	"<D{,ND(,>D}"
-
-#include "gmt_dev.h"
-
-#define GMT_PROG_OPTIONS "-Vhio" GMT_ADD_x_OPT
-
-#define TOL		1.0e-7
-#define GAMMA 		6.673e-11	/* Gravitational constant */
-#define G               9.81            /* Normal gravity */
-#define SI_TO_MGAL	1.0e5		/* Convert m/s^2 to mGal */
-#define SI_TO_EOTVOS	1.0e9		/* Convert (m/s^2)/m to Eotvos */
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "-Vdehio" GMT_ADD_x_OPT
 
 struct TALWANI2D_CTRL {
 	struct Out {	/* -> */
@@ -66,9 +63,10 @@ struct TALWANI2D_CTRL {
 		bool active;
 		double rho;
 	} D;
-	struct F {	/* -F[f|n|v] */
+	struct F {	/* -F[f|n[<lat>]|v] */
 		bool active;
 		unsigned int mode;
+		double lat;
 	} F;
 	struct M {	/* -Mh|z  */
 		bool active[2];	/* True if km, else m */
@@ -113,6 +111,8 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	
 	/* Initialize values whose defaults are not 0/false/NULL */
 
+	C->F.lat = 45.0;	/* So we compute normal gravity at 45 */
+	
 	return (C);
 }
 
@@ -152,7 +152,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct TALWANI2D_CTRL *Ctrl, struct G
 				Ctrl->F.active = true;
 				switch (opt->arg[0]) {
 					case 'v': Ctrl->F.mode = TALWANI2D_VGG;		break;
-					case 'n': Ctrl->F.mode = TALWANI2D_GEOID;	break;
+					case 'n': Ctrl->F.mode = TALWANI2D_GEOID;
+						if (opt->arg[1]) Ctrl->F.lat = atof (&opt->arg[1]);
+						break;
 					default:  Ctrl->F.mode = TALWANI2D_FAA; 	break;
 				}
 				break;
@@ -198,6 +200,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct TALWANI2D_CTRL *Ctrl, struct G
 				break;
 		}
 	}
+	n_errors += gmt_M_check_condition (GMT, fabs (Ctrl->F.lat) > 90.0,
+	                                 "Syntax error -Fn option: Latitude out of range\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && Ctrl->N.active,
 	                                 "Syntax error -N option: Cannot also specify -T\n");
 	n_errors += gmt_M_check_condition (GMT, (Ctrl->Z.mode & 2) && Ctrl->Z.ymin >= Ctrl->Z.ymax,
@@ -212,9 +216,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct TALWANI2D_CTRL *Ctrl, struct G
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: talwani2d <modelfile> [-A] [-D<rho>] [-Ff|n|v]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: talwani2d <modelfile> [-A] [-D<rho>] [-Ff|n[<lat>]|v]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t[-M[hz]] [-N<trktable>] [-T[<xmin>/<xmax>/<xinc>[+]]] [%s] [-Z[<level>][/<ymin/<ymax>]]\n", GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE,"\t[%s]\n\t[%s] [%s]%s\n\n", GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_x_OPT);
+	GMT_Message (API, GMT_TIME_NONE,"\t[%s] [%s] [%s]\n\t[%s] [%s]%s\n\n", GMT_d_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_x_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -224,7 +228,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Sets fixed density contrast that overrides settings in model file (in kg/m^3).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Specify desired geopotential field component:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   f = FAA Free-air anomalies (mGal) [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   n = Geoid anomalies (meter).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   n = Geoid anomalies (meter).  Optionally append <lat> for evaluation of normal gravity [45].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   v = VGG Vertical Gravity Gradient anomalies (Eotvos = 0.1 mGal/km).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-M sets units used, as follows:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Mh indicates all x-distances are given in km [meters]\n");
@@ -237,7 +241,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Z Set observation level for output locations [0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   For FAA only: Optionally append <ymin/ymax> to get a 2.5-D solution.\n");
-	GMT_Option (API, "h,i,o,x,.");
+	GMT_Option (API, "d,e,h,i,o,x,.");
 	return (GMT_MODULE_USAGE);
 }
 
@@ -316,7 +320,7 @@ GMT_LOCAL double grav_2_5D (struct GMT_CTRL *GMT, double x[], double z[], unsign
 		xx0 = xx1;
 		zz0 = zz1;
 	}
-	sum *= GAMMA * rho * SI_TO_MGAL;	/* To get mGal */
+	sum *= SI_GAMMA * rho * SI_TO_MGAL;	/* To get mGal */
 	return (sum);
 }
 
@@ -356,7 +360,7 @@ GMT_LOCAL double get_grav2d (struct GMT_CTRL *GMT, double x[], double z[], unsig
 		ri = ri1;
 		phi_i = phi_i1;
 	}
-	sum *= 2.0 * GAMMA * rho * SI_TO_MGAL;	/* To get mGal */
+	sum *= 2.0 * SI_GAMMA * rho * SI_TO_MGAL;	/* To get mGal */
 	return (sum);
 }
 
@@ -397,11 +401,11 @@ GMT_LOCAL double get_vgg2d (struct GMT_CTRL *GMT, double *x, double *z, unsigned
 		}
 	}
 
-	sum *= (-GAMMA * rho * SI_TO_EOTVOS);        /* To get Eotvos */
+	sum *= (-SI_GAMMA * rho * SI_TO_EOTVOS);        /* To get Eotvos */
 	return (sum);
 }
 
-GMT_LOCAL double get_geoid2d (struct GMT_CTRL *GMT, double y[], double z[], unsigned int n, double y0, double z0, double rho) {
+GMT_LOCAL double get_geoid2d (struct GMT_CTRL *GMT, double y[], double z[], unsigned int n, double y0, double z0, double rho, double G0) {
 	/* Based on Chaptman, 1979, Techniques for interpretation of geoid anomalies, JGR, 84 (B8), 3793-3801.
 	 *  y0;		Y-coordinate of observation point, in m
 	 *  z0;		Z-coordinate of observation point, in m
@@ -409,6 +413,7 @@ GMT_LOCAL double get_geoid2d (struct GMT_CTRL *GMT, double y[], double z[], unsi
 	 *  z[];	Z-coordinates of vertices, in m
 	 *  n;	Number of vertices, with 1st == last point
 	 *  rho;	Density contrast, in kg/m3
+	 *   G0;	Normal gravity at suitable latitude
 	 */
 	int i1, i2;
 	double dy1, dy2, dz1, dz2, hyp1, hyp2, mi, mi2, a2, bi, ci, ci2, di, di2;
@@ -484,11 +489,11 @@ GMT_LOCAL double get_geoid2d (struct GMT_CTRL *GMT, double y[], double z[], unsi
 		}
 		N = N + ni;
 	}
-	N *= (-GAMMA * rho / G);
+	N *= (-SI_GAMMA * rho / G0);
 	return (N);
 }
 
-GMT_LOCAL double get_one_output2D (struct GMT_CTRL *GMT, double x_obs, double z_obs, struct BODY2D *body, unsigned int n_bodies, unsigned int mode, double ymin, double ymax) {
+GMT_LOCAL double get_one_output2D (struct GMT_CTRL *GMT, double x_obs, double z_obs, struct BODY2D *body, unsigned int n_bodies, unsigned int mode, double ymin, double ymax, double G0) {
 	/* Evaluate output at a single observation point (x,z) */
 	/* Work array vtry must have at least of length ndepths */
 	unsigned int k;
@@ -504,7 +509,7 @@ GMT_LOCAL double get_one_output2D (struct GMT_CTRL *GMT, double x_obs, double z_
 		else if (mode == TALWANI2D_VGG) /* VGG */
 			v += get_vgg2d (GMT, body[k].x, body[k].z, body[k].n, x_obs, z_obs, body[k].rho);
 		else /* GEOID*/
-			v += get_geoid2d (GMT, body[k].x, body[k].z, body[k].n, x_obs, z_obs, body[k].rho);
+			v += get_geoid2d (GMT, body[k].x, body[k].z, body[k].n, x_obs, z_obs, body[k].rho, G0);
 		if (area < 0.0) v = -v;	/* Polygon went counter-clockwise */
 		v_sum += v;
 	}
@@ -518,8 +523,8 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 	int srow, error = 0, ns;
 	unsigned int k, tbl, seg, n = 0, geometry, n_bodies, dup_node = 0, n_duplicate = 0;
 	size_t n_alloc = 0, n_alloc1 = 0;
-	uint64_t dim[4] = {1, 1, 0, 2}, row;
-	double scl, rho = 0.0, z_level, answer, min_answer = DBL_MAX, max_answer = -DBL_MAX;
+	uint64_t dim[GMT_DIM_SIZE] = {1, 1, 0, 2}, row;
+	double scl, rho = 0.0, G0, z_level, answer, min_answer = DBL_MAX, max_answer = -DBL_MAX;
 	bool first = true;
 	
 	char *uname[2] = {"meter", "km"}, *kind[4] = {"FAA", "VGG", "GEOID", "FAA(2.5-D)"};
@@ -547,8 +552,8 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 
 	/* Parse the command-line arguments */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
@@ -584,14 +589,14 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 		for (row = 0; row < dim[GMT_ROW]; row++) S->data[GMT_X][row] = (row == (S->n_rows-1)) ? Ctrl->T.max: Ctrl->T.min + row * Ctrl->T.inc;
 	}
 	else {	/* Got a dataset with output locations */
-		geometry = GMT_IS_PLP;	/* We dont really know */
+		geometry = GMT_IS_PLP;	/* We don't really know */
 		gmt_disable_i_opt (GMT);	/* Do not want any -i to affect the reading from -C,-F,-L files */
 		if ((Out = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_PLP, GMT_READ_NORMAL, NULL, Ctrl->N.file, NULL)) == NULL) {
 			Return (API->error);
 		}
-		if (Out->n_columns < 2) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Input file %s has %d column(s) but at least 2 are needed\n", Ctrl->N.file, (int)Out->n_columns);
-			Return (GMT_DIM_TOO_SMALL);
+		if (Out->n_columns < 2) {	/* Only got x, must allocate output space for prediction in the 2nd column */
+			gmt_adjust_dataset (GMT, Out, 2U);
+			geometry = GMT_IS_LINE;	/* Since we are making from scratch */
 		}
 		gmt_reenable_i_opt (GMT);	/* Recover settings provided by user (if -i was used at all) */
 	}
@@ -620,7 +625,7 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 	
 	/* Read the sliced model */
 	do {	/* Keep returning records until we reach EOF */
-		if ((in = GMT_Get_Record (API, GMT_READ_DOUBLE, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+		if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 			if (gmt_M_rec_is_error (GMT)) { 		/* Bail if there are any read errors */
 				gmt_M_free (GMT, body);
 				Return (GMT_RUNTIME_ERROR);
@@ -664,6 +669,7 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 				}
 				continue;
 			}
+			assert (in != NULL);						/* Should never get here */
 		}
 		/* Clean data record to process.  Add point unless duplicate */
 		if (Ctrl->A.active) in[GMT_Y] = -in[GMT_Y];
@@ -672,6 +678,24 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 			dup_node = n;
 		}
 		else {
+			if (first) {	/* Had no header record at all */
+				if (!Ctrl->D.active) {
+					GMT_Report (API, GMT_MSG_VERBOSE, "Found no segment header and -D not set - must quit\n");
+					gmt_M_free (GMT, body);
+					Return (API->error);
+				}
+				first = false;
+				if (Ctrl->D.active) rho = Ctrl->D.rho;
+				/* Allocate array for this body */
+				n_alloc = GMT_CHUNK;
+				x = gmt_M_memory (GMT, NULL, n_alloc, double);
+				z = gmt_M_memory (GMT, NULL, n_alloc, double);
+				n = 0;
+				if (n_bodies == n_alloc1) {
+					n_alloc1 <<= 1;
+					body = gmt_M_memory (GMT, body, n_alloc1, struct BODY2D);
+				}
+			}
 			x[n] = in[GMT_X];	z[n] = in[GMT_Y];
 			if (Ctrl->M.active[TALWANI2D_HOR]) x[n] *= METERS_IN_A_KM;	/* Change distances to m */
 			if (Ctrl->M.active[TALWANI2D_VER]) z[n] *= METERS_IN_A_KM;	/* Change distances to m */
@@ -699,6 +723,7 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 	if (n_duplicate) GMT_Report (API, GMT_MSG_VERBOSE, "Ignored %u duplicate vertices\n", n_duplicate);
 	
 	if (Ctrl->A.active) Ctrl->Z.level = -Ctrl->Z.level;
+	G0 = g_normal (Ctrl->F.lat);
 	
 	/* Now we can write to the screen the user's polygon model characteristics. */
 	
@@ -714,11 +739,11 @@ int GMT_talwani2d (void *V_API, int mode, void *args) {
 			S = Out->table[tbl]->segment[seg];	/* Current segment */
 #ifdef _OPENMP
 			/* Spread calculation over selected cores */
-#pragma omp parallel for private(srow,z_level,answer) shared(GMT,Ctrl,S,scl,body,n_bodies)
+#pragma omp parallel for private(srow,z_level,answer) shared(GMT,Ctrl,S,scl,body,n_bodies, G0)
 #endif
 			for (srow = 0; srow < (int)S->n_rows; srow++) {	/* Calculate attraction at all output locations for this segment. OpenMP requires sign int srow */
 				z_level = (S->n_columns == 2 && !(Ctrl->Z.mode & 1)) ? S->data[GMT_Y][srow] : Ctrl->Z.level;
-				answer = get_one_output2D (GMT, S->data[GMT_X][srow] * scl, z_level, body, n_bodies, Ctrl->F.mode, Ctrl->Z.ymin, Ctrl->Z.ymax);
+				answer = get_one_output2D (GMT, S->data[GMT_X][srow] * scl, z_level, body, n_bodies, Ctrl->F.mode, Ctrl->Z.ymin, Ctrl->Z.ymax, G0);
 				S->data[GMT_Y][srow] = answer;
 				if (answer < min_answer) min_answer = answer;
 				if (answer > max_answer) max_answer = answer;
